/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */

/* nautilus-samba-utilities.h - utilities for dealing with Samba sharing.

   Copyright (C) 2002 Ximian, Inc.

   The Gnome Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   The Gnome Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with the Gnome Library; see the file COPYING.LIB.  If not,
   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.

   Authors: Hans Petter Jansson <hpj@ximian.com>
            Dave Camp <dave@ximian.com>
*/

#include <config.h>

#include <string.h>

#include <eel/eel-vfs-extensions.h>

#include "nautilus-samba-utilities.h"

typedef struct {
	char *uri;
	char *name;
	char *comment;
} NautilusSambaShare;


static GHashTable *share_hash = NULL;
static EelReadFileHandle *conf_read_handle = NULL;

static void
nautilus_samba_share_free (NautilusSambaShare *share) 
{
	g_free (share->uri);
	g_free (share->name);
	g_free (share->comment);
	g_free (share);
}

static NautilusSambaShare *
start_section (const char *name)
{
	NautilusSambaShare *share;

	if (!strcmp (name, "globals")) {
		return NULL;
	}

	share = g_new0 (NautilusSambaShare, 1);
	share->name = g_strdup (name);
	
	return share;
}

static GList *
finish_section (GList *sections, NautilusSambaShare *share)
{
	if (share) {
		if (share->uri) {
			sections = g_list_prepend (sections, share);
		} else {
			nautilus_samba_share_free (share);
		}
	}
	return sections;
}

static void
process_key (NautilusSambaShare *share, 
	     const char *key,
	     const char *value)
{
	if (!strcmp (key, "path")) {
		share->uri = eel_make_uri_canonical (value);
	} else if (!strcmp (key, "comment")) {
		share->comment = g_strdup (value);
	}
}

static GList *
parse_conf (const char *contents)
{
	GList *sections;
	NautilusSambaShare *share;
	const char *begin;
	const char *end;
	char *line_copy;
	char *line;
	char **split_line;
	char *section_name;
	char *key;
	char *value;

	sections = NULL;
	share = NULL;
	begin = contents;
	while (begin && begin[0]) {
		end = strchr (begin, '\n');
		if (end) {
			line_copy = g_strndup (begin, end - begin);
		} else {
			line_copy = g_strdup (begin);
		}

		line = g_strstrip (line_copy);

		if (!(line[0] == ';' ||
		      line[0] == '#' ||
		      line[0] == '\0')) { /* Comments */
			if (line[0] == '[' && line[strlen (line) - 1] == ']') {
				section_name = g_strdup (line + 1);
				section_name[strlen (section_name) - 1] = '\0';

				sections = finish_section (sections, share);
				share = start_section (section_name);
			} else if (strchr (line, '=')) {
				split_line = g_strsplit (line, "=", 2);
				if (split_line[0]) {
					key = split_line[0];
					if (split_line[1]) {
						value = split_line[1];
					} else {
						value = "";
					}

					key = g_strstrip (key);
					value = g_strstrip (value);

					if (share) {
						process_key (share, 
							     key, 
							     value);
					}
				}
				g_strfreev (split_line);
			}
		}

		g_free (line_copy);

		if (end) {
			begin = end + 1;
		} else {
			begin = NULL;
		}
	}
	sections = finish_section (sections, share);

	return sections;
}


static void
conf_read_cb (GnomeVFSResult result,
	      GnomeVFSFileSize file_size,
	      char *file_contents,
	      gpointer callback_data)
{
	GHashTable *new_share_hash;
	NautilusSambaShare *share;
	GList *shares;
	GList *l;

	conf_read_handle = NULL;
	
	if (result == GNOME_VFS_OK) {
		new_share_hash = g_hash_table_new_full 
			(g_str_hash,
			 g_str_equal,
			 NULL,
			 (GDestroyNotify)nautilus_samba_share_free);

		shares = parse_conf (file_contents);
		for (l = shares; l != NULL; l = l->next) {
			share = l->data;
			g_hash_table_insert (new_share_hash, 
					     share->uri, 
					     share);
		}
		g_list_free (shares);
		if (share_hash) {
			g_hash_table_destroy (share_hash);
		}
		share_hash = new_share_hash;
	} else {
		if (share_hash) {
			g_hash_table_destroy (share_hash);
			share_hash = NULL;
		}
	}
}

static void
reread_samba_config (void)
{
	if (conf_read_handle) {
		eel_read_file_cancel (conf_read_handle);
	}
	
	conf_read_handle = eel_read_entire_file_async
		("file:///etc/samba/smb.conf",
		 GNOME_VFS_PRIORITY_MIN,
		 conf_read_cb,
		 NULL);
}

gboolean
nautilus_samba_uri_is_shared (const char *uri)
{
	if (share_hash) {
		return (g_hash_table_lookup (share_hash, uri) != NULL);
	} else {
		return FALSE;
	}
}

void
nautilus_samba_init (void)
{
	static gboolean initialized = FALSE;
	
	if (!initialized) {
		initialized = TRUE;
		
		reread_samba_config ();
	}
}
